#
# The MIT License (MIT)
# Copyright (c) 2021 M.O.S.S. Computer Grafik Systeme GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the Software
# is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT,TORT OR OTHERWISE, ARISING FROM, OUT
# OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

import json
import logging

logger = logging.getLogger(__file__)


class EmsCatalogException(Exception):
    """
    Base catalog exception
    """


class EmsCatalog:
    """
    Base catalog class
    """

    def __init__(self, session, name, id=None, desc=None, entries=None, fields=None):
        self.name = name
        self.id = id
        self.description = desc
        self.data = self._read_catalog_values(entries)
        self.fields = fields

        self._session = session

    def _read_catalog_values(self, entries):
        """
        This fn is used to convert the catalog entries into a simple dictionary.
        :param entries: The ID of the feature.
        :type entries: int
        :returns:  dict -- a dictionary of catalog values, None on error.
        :raises: EmsCatalogException
        """
        data = {}
        try:
            for entry in entries:
                if "DCKEY" not in entry:
                    raise EmsCatalogException("Invalid catalog: DCKEY is missing")
                if "DCTXT" not in entry:
                    raise EmsCatalogException("Invalid catalog: DCTXT is missing")
            try:
                # data[entry["ID"]] = entry["DCTXT"]
                data = entries
            except KeyError:
                raise EmsCatalogException("Invalid catalog: DCKEY or DCTXT are missing")
        except TypeError:  # "'NoneType' object is not iterable" if entries is None
            return None

        return data

    def __getitem__(self, key):
        """
        Return the value of the catalog
        """
        logger.debug("Querying catalog %s using key %s", self.name, key)

        if key is None:
            logger.warning("Value is None. Ignoring catalog.")
            return key

        try:
            catalog_value = self.data[key]
        except KeyError:
            logger.warning("The key %s does not exist in catalog %s", key, self.name)
            catalog_value = key
        except IndexError:
            logger.error("Error reading catalog %s key %s", self, key)
            catalog_value = key

        # With some changes in code the returned value can be a dictionary.
        # Avoid this by doing another get. TODO Remove it and fix everything.

        if isinstance(catalog_value, dict):
            logger.debug("The value is a dictionary")
            return catalog_value.get("DCTXT", None)

        return catalog_value

    def add_entries(self, project_name, catalog_name, entries):
        """Add entries for a catalog of a project.

        Args:
            project_name: the project name
            catalog_name: the catalog name
            entries: a list of entries
        """

        url = "/".join(["rest", project_name, catalog_name.upper(), "addEntries"])

        request_parameters = {
            "f": "json",
            "entries": json.dumps(entries),
        }

        response_dict = self._ems_request(url, request_parameters, "POST")
        response_result = response_dict.get("addEntriesResult")

        if response_result is not None:
            response_success = response_result[0].get("success")

            if response_success:
                logger.info("Entry added correctly")
                return True
            else:
                logger.error("Error adding entry")
                return False

        else:
            raise EmsCatalogException("Invalid response from the server")

        return self

    def _ems_request(self, url, parameters, method="GET"):
        """
        Return a json response from a session
        """
        logger.debug("New request to %s parameters %s", url, parameters)

        if method == "POST":
            response = self._session.post(url, data=parameters)
        else:
            response = self._session.get(url, params=parameters)

        if response.status_code != 200:
            logger.error(
                "Error talking with ems. Url: %s Parameters: %s Status code: %s",
                url,
                parameters,
                response.status_code,
            )
            raise EmsCatalogException("Error talking with EMS")
        try:
            json_response = response.json()
        except ValueError as exception:
            logger.exception(
                "Response is not json. You are probably on the login website:%s",
                exception,
            )
        else:
            return json_response

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name



#
# The MIT License (MIT)
# Copyright (c) 2021 M.O.S.S. Computer Grafik Systeme GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the Software
# is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT,TORT OR OTHERWISE, ARISING FROM, OUT
# OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

import logging

from requests import Session
from requests.compat import urljoin


logger = logging.getLogger(__file__)


class SessionBaseUrl(Session):
    """
    This subclass will allow to store the base address
    of the service. This will avoid to always pass the base url.
    """

    def __init__(self, base_url, token=None):
        self.base_url = base_url
        # self.token = token
        super(SessionBaseUrl, self).__init__()

    def request(self, method, url, *args, **kwargs):
        """Send a Rest request to the url with args and additional kwargs

        :param method: "GET" or "POST"
        :type method: str
        :param url: The url to send the request to
        :type url: str
        :param args: a variable number of arguments for the request
        :type args: args
        :param kwargs: a keyworded, variable-length argument list for the request
        :type kwargs: kwargs
        :returns: The html response dict for the request
        :raises:
        """
        url = urljoin(self.base_url, url)

        # if "params" in kwargs:
        #    kwargs["params"] = {**kwargs["params"], "token": self.token}

        return super(SessionBaseUrl, self).request(method, url, *args, **kwargs)



#
# The MIT License (MIT)
# Copyright (c) 2021 M.O.S.S. Computer Grafik Systeme GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the Software
# is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT,TORT OR OTHERWISE, ARISING FROM, OUT
# OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__path__ = __import__("pkgutil").extend_path(__path__, __name__)



#
# The MIT License (MIT)
# Copyright (c) 2021 M.O.S.S. Computer Grafik Systeme GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the Software
# is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT,TORT OR OTHERWISE, ARISING FROM, OUT
# OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

import logging

LOGGER = logging.getLogger(__file__)


class EmsQueryResult:
    """
    The result of an EMS query
    """

    def __init__(self, session, project_name, objectclass_name, *args):

        self._session = session
        self._project = project_name
        self._objectclass_name = objectclass_name
        self._id = 0

        for key, value in args[0].items():
            # setattr(self, key.lower(), value)   Warum lower()?
            setattr(self, key, value)

    def variants(self, feature_id):
        """Return all the variants for a given feature ID

        Args:
            feature_id: The feature ID to get the variants for

        Returns:
            variants: a list of all variants or None
        """
        url = "/".join(
            ["rest", self._project, self._objectclass_name, str(feature_id), "variants"]
        )
        parameters = {"f": "json"}

        LOGGER.debug("Getting variants from %s", url)

        response = self._session.get(url, params=parameters)
        try:
            response_dict = response.json()
        except KeyError:
            return None

        LOGGER.debug(response_dict)
        try:
            variants = response_dict["variants"]
            return variants
        except KeyError:
            error_dictionary = response_dict["error"]
            LOGGER.error("Message: %s", error_dictionary["message"])
            LOGGER.error("Description: %s", error_dictionary["description"])
            LOGGER.error("Code: %s", error_dictionary["code"])
            return None
        else:
            return None

    def derive(self, feature_id, variant_id, name, description, status=0, category=0):
        """Derive a new variant from an existing variant

        Args:
            feature_id: The feature ID of the existing vaiant
            variant_id: The variant ID of the existing vaiant
            name: The new variant name
            description: The new description
            status: the new status
            category: the new category

        Returns:
            response_dict: the response dict to the derive request or None
        """

        url = "/".join(
            [
                "rest",
                self._project,
                self._objectclass_name,
                str(feature_id),
                "variant",
                str(variant_id),
                "derive",
            ]
        )
        parameters = {
            "f": "json",
            "name": name,
            "description": description,
            "status": status,
            "category": category,
        }
        response = self._session.post(url, data=parameters)
        try:
            response_dict = response.json()
        except KeyError:
            return None

        try:
            if response_dict["deriveVariantResult"]["success"] is False:
                error_dictionary = response_dict["deriveVariantResult"]["error"]

                LOGGER.error("Message: %s", error_dictionary["message"])
                LOGGER.error("Description: %s", error_dictionary["description"])
                LOGGER.error("Code: %s", error_dictionary["code"])
                return None
        except KeyError:
            error_dictionary = response_dict["error"]
            LOGGER.error("Message: %s", error_dictionary["message"])
            LOGGER.error("Description: %s", error_dictionary["description"])
            LOGGER.error("Code: %s", error_dictionary["code"])
            return None

        LOGGER.info(
            "Successfully derived new variant '%s' from '%s'",
            response_dict["deriveVariantResult"]["objectId"],
            feature_id,
        )
        return response_dict

    def delete(self, feature_id, variant_id):
        """Delete a variant

        Args:
            feature_id: The feature ID of the existing variant
            variant_id: The variant ID of the existing variant

        Returns:
            response_dict: the response dict to the delete request or None
        """
        url = "/".join(
            [
                "rest",
                self._project,
                self._objectclass_name,
                str(feature_id),
                "variant",
                str(variant_id),
                "delete",
            ]
        )
        parameters = {"f": "json"}
        response = self._session.post(url, data=parameters)
        try:
            response_dict = response.json()
        except KeyError:
            return None

        try:
            if response_dict["deleteVariantResult"]["success"] is False:
                error_dictionary = response_dict["deleteVariantResult"]["error"]
                LOGGER.error("Message: %s", error_dictionary["message"])
                LOGGER.error("Description: %s", error_dictionary["description"])
                LOGGER.error("Code: %s", error_dictionary["code"])
                return False
        except KeyError:
            error_dictionary = response_dict["error"]
            LOGGER.error("Message: %s", error_dictionary["message"])
            LOGGER.error("Description: %s", error_dictionary["description"])
            LOGGER.error("Code: %s", error_dictionary["code"])
            return False

        LOGGER.info(
            "Successfully deleted variant '%s' from '%s'.",
            str(feature_id),
            str(variant_id),
        )
        return True

    def drain(self, feature_id, variant_id, object_classes):
        """Drains a variant from a list of object classes

        Args:
            feature_id: The feature ID to drain from
            variant_id: The variant ID to drain from
            object_classes: A list of objectclasses to drain from

        Returns:
            response_dict: the response dict to the drain request or None
        """

        url = "/".join(
            [
                "rest",
                self._project,
                self._objectclass_name,
                str(feature_id),
                "variant",
                str(variant_id),
                "drain",
            ]
        )
        parameters = {"f": "json", "objectClasses": object_classes}
        response = self._session.post(url, data=parameters)
        try:
            response_dict = response.json()
        except KeyError:
            return None

        try:
            if response_dict["drainVariantResult"]["success"] is False:
                error_dictionary = response_dict["drainVariantResult"]["error"]
                LOGGER.error("Message: %s", error_dictionary["message"])
                LOGGER.error("Description: %s", error_dictionary["description"])
                LOGGER.error("Code: %s", error_dictionary["code"])
                return False
        except KeyError:
            error_dictionary = response_dict["error"]
            LOGGER.error("Message: %s", error_dictionary["message"])
            LOGGER.error("Description: %s", error_dictionary["description"])
            LOGGER.error("Code: %s", error_dictionary["code"])
            return False

        LOGGER.info(
            "Successfully drained variant '%s' from object classes (%s).",
            response_dict["drainVariantResult"]["objectId"],
            object_classes,
        )
        return True



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['moss', 'moss.ems', 'moss.ems.utilities']

package_data = \
{'': ['*']}

install_requires = \
['anytree>=2.8.0,<3.0.0', 'requests>=2.18.0,<3.0.0']

extras_require = \
{':python_version < "3.5"': ['typing>=3.7.4,<4.0.0']}

setup_kwargs = {
    'name': 'moss-emspy',
    'version': '1.0.3',
    'description': 'Package to interact with MOSS WEGA-EMS',
    'long_description': '# MOSS emspy\n\n## Description\n\nThis is the Python SDK to interact with [M.O.S.S. Computer Grafik Systeme GmbH](https://www.moss.de/wega/) WEGA-EMS\n\n## Installation\n\nThis package kann be installed using pip\n\n```shell\npython -m pip install moss_emspy\n```\n\n## Usage\n\n```python\nmy_service = Service("http://localhost:8080/wega-ems/",\n            username="Test",\n            password="Test")\nmy_service.projects\n```\n',
    'author': 'M.O.S.S.Computer Grafik Systeme GmbH',
    'author_email': 'develop@moss.de',
    'maintainer': 'M.O.S.S.Computer Grafik Systeme GmbH',
    'maintainer_email': 'develop@moss.de',
    'url': 'https://www.moss.de/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
}


setup(**setup_kwargs)
